@@ -1,14 +1,9 @@
-// Copyright (C) 2008-2011 NICTA (www.nicta.com.au)
-// Copyright (C) 2008-2011 Conrad Sanderson
+// Copyright (C) 2008-2013 Conrad Sanderson
+// Copyright (C) 2008-2013 NICTA (www.nicta.com.au)
 // 
-// This file is part of the Armadillo C++ library.
-// It is provided without any warranty of fitness
-// for any purpose. You can redistribute this file
-// and/or modify it under the terms of the GNU
-// Lesser General Public License (LGPL) as published
-// by the Free Software Foundation, either version 3
-// of the License or (at your option) any later version.
-// (see http://www.opensource.org/licenses for more info)
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 
 //! \addtogroup Col
@@ -19,11 +14,21 @@
 template<typename eT>
 inline
 Col<eT>::Col()
-  : Mat<eT>(0, 1)
+  : Mat<eT>(arma_vec_indicator(), 1)
+  {
+  arma_extra_debug_sigprint();
+  }
+
+
+
+template<typename eT>
+inline
+Col<eT>::Col(const Col<eT>& X)
+  : Mat<eT>(arma_vec_indicator(), X.n_elem, 1, 1)
   {
   arma_extra_debug_sigprint();
   
-  access::rw(Mat<eT>::vec_state) = 1;
+  arrayops::copy((*this).memptr(), X.memptr(), X.n_elem);
   }
 
 
@@ -32,11 +37,9 @@ Col<eT>::Col()
 template<typename eT>
 inline
 Col<eT>::Col(const uword in_n_elem)
-  : Mat<eT>(in_n_elem, 1)
+  : Mat<eT>(arma_vec_indicator(), in_n_elem, 1, 1)
   {
   arma_extra_debug_sigprint();
-  
-  access::rw(Mat<eT>::vec_state) = 1;
   }
 
 
@@ -44,12 +47,39 @@ Col<eT>::Col(const uword in_n_elem)
 template<typename eT>
 inline
 Col<eT>::Col(const uword in_n_rows, const uword in_n_cols)
+  : Mat<eT>(arma_vec_indicator(), 0, 0, 1)
   {
   arma_extra_debug_sigprint();
   
-  access::rw(Mat<eT>::vec_state) = 1;
+  Mat<eT>::init_warm(in_n_rows, in_n_cols);
+  }
+
+
+
+template<typename eT>
+template<typename fill_type>
+inline
+Col<eT>::Col(const uword in_n_elem, const fill::fill_class<fill_type>& f)
+  : Mat<eT>(arma_vec_indicator(), in_n_elem, 1, 1)
+  {
+  arma_extra_debug_sigprint();
+  
+  (*this).fill(f);
+  }
+
+
+
+template<typename eT>
+template<typename fill_type>
+inline
+Col<eT>::Col(const uword in_n_rows, const uword in_n_cols, const fill::fill_class<fill_type>& f)
+  : Mat<eT>(arma_vec_indicator(), 0, 0, 1)
+  {
+  arma_extra_debug_sigprint();
   
   Mat<eT>::init_warm(in_n_rows, in_n_cols);
+  
+  (*this).fill(f);
   }
 
 
@@ -132,47 +162,162 @@ Col<eT>::operator=(const std::string& text)
 
 
 
-#if defined(ARMA_USE_CXX11)
-
+//! create a column vector from std::vector
 template<typename eT>
 inline
-Col<eT>::Col(const std::initializer_list<eT>& list)
+Col<eT>::Col(const std::vector<eT>& x)
+  : Mat<eT>(arma_vec_indicator(), uword(x.size()), 1, 1)
   {
-  arma_extra_debug_sigprint();
+  arma_extra_debug_sigprint_this(this);
   
-  access::rw(Mat<eT>::vec_state) = 2;
-  
-  Mat<eT>::operator=(list);
-  
-  std::swap( access::rw(Mat<eT>::n_rows), access::rw(Mat<eT>::n_cols) );
-  
-  access::rw(Mat<eT>::vec_state) = 1;
+  if(x.size() > 0)
+    {
+    arrayops::copy( Mat<eT>::memptr(), &(x[0]), uword(x.size()) );
+    }
   }
-
-
-
+  
+  
+  
+//! create a column vector from std::vector
 template<typename eT>
 inline
 const Col<eT>&
-Col<eT>::operator=(const std::initializer_list<eT>& list)
+Col<eT>::operator=(const std::vector<eT>& x)
   {
   arma_extra_debug_sigprint();
   
-  access::rw(Mat<eT>::vec_state) = 2;
+  Mat<eT>::init_warm(uword(x.size()), 1);
   
-  Mat<eT>::operator=(list);
-  
-  std::swap( access::rw(Mat<eT>::n_rows), access::rw(Mat<eT>::n_cols) );
-  
-  access::rw(Mat<eT>::vec_state) = 1;
+  if(x.size() > 0)
+    {
+    arrayops::copy( Mat<eT>::memptr(), &(x[0]), uword(x.size()) );
+    }
   
   return *this;
   }
 
+
+
+#if defined(ARMA_USE_CXX11)
+  
+  template<typename eT>
+  inline
+  Col<eT>::Col(const std::initializer_list<eT>& list)
+    {
+    arma_extra_debug_sigprint();
+    
+    access::rw(Mat<eT>::vec_state) = 2;
+    
+    Mat<eT>::operator=(list);
+    
+    std::swap( access::rw(Mat<eT>::n_rows), access::rw(Mat<eT>::n_cols) );
+    
+    access::rw(Mat<eT>::vec_state) = 1;
+    }
+  
+  
+  
+  template<typename eT>
+  inline
+  const Col<eT>&
+  Col<eT>::operator=(const std::initializer_list<eT>& list)
+    {
+    arma_extra_debug_sigprint();
+    
+    access::rw(Mat<eT>::vec_state) = 2;
+    
+    Mat<eT>::operator=(list);
+    
+    std::swap( access::rw(Mat<eT>::n_rows), access::rw(Mat<eT>::n_cols) );
+    
+    access::rw(Mat<eT>::vec_state) = 1;
+    
+    return *this;
+    }
+  
+  
+  
+  template<typename eT>
+  inline
+  Col<eT>::Col(Col<eT>&& X)
+    : Mat<eT>(arma_vec_indicator(), 1)
+    {
+    arma_extra_debug_sigprint(arma_boost::format("this = %x   X = %x") % this % &X);
+    
+    access::rw(Mat<eT>::n_rows) = X.n_rows;
+    access::rw(Mat<eT>::n_cols) = 1;
+    access::rw(Mat<eT>::n_elem) = X.n_elem;
+    
+    if( ((X.mem_state == 0) && (X.n_elem > arma_config::mat_prealloc)) || (X.mem_state == 1) || (X.mem_state == 2) )
+      {
+      access::rw(Mat<eT>::mem_state) = X.mem_state;
+      access::rw(Mat<eT>::mem)       = X.mem;
+      
+      access::rw(X.n_rows)    = 0;
+      access::rw(X.n_cols)    = 1;
+      access::rw(X.n_elem)    = 0;
+      access::rw(X.mem_state) = 0;
+      access::rw(X.mem)       = 0;
+      }
+    else
+      {
+      (*this).init_cold();
+      
+      arrayops::copy( (*this).memptr(), X.mem, X.n_elem );
+      
+      if( (X.mem_state == 0) && (X.n_elem <= arma_config::mat_prealloc) )
+        {
+        access::rw(X.n_rows) = 0;
+        access::rw(X.n_cols) = 1;
+        access::rw(X.n_elem) = 0;
+        access::rw(X.mem)    = 0;
+        }
+      }
+    }
+  
+  
+  
+  template<typename eT>
+  inline
+  const Col<eT>&
+  Col<eT>::operator=(Col<eT>&& X)
+    {
+    arma_extra_debug_sigprint(arma_boost::format("this = %x   X = %x") % this % &X);
+    
+    (*this).steal_mem(X);
+    
+    if( (X.mem_state == 0) && (X.n_elem <= arma_config::mat_prealloc) && (this != &X) )
+      {
+      access::rw(X.n_rows) = 0;
+      access::rw(X.n_cols) = 1;
+      access::rw(X.n_elem) = 0;
+      access::rw(X.mem)    = 0;
+      }
+    
+    return *this;
+    }
+  
 #endif
 
 
 
+template<typename eT>
+inline
+Col<eT>::Col(const SpCol<eT>& X)
+  : Mat<eT>(arma_vec_indicator(), X.n_elem, 1, 1)
+  {
+  arma_extra_debug_sigprint_this(this);
+
+  arrayops::inplace_set(Mat<eT>::memptr(), eT(0), X.n_elem);
+
+  for(typename SpCol<eT>::const_iterator it = X.begin(); it != X.end(); ++it)
+    {
+    at(it.row()) = (*it);
+    }
+  }
+
+
+
 template<typename eT>
 inline
 const Col<eT>&
@@ -188,13 +333,26 @@ Col<eT>::operator=(const eT val)
 
 
 template<typename eT>
-template<typename T1>
 inline
-Col<eT>::Col(const Base<eT,T1>& X)
+const Col<eT>&
+Col<eT>::operator=(const Col<eT>& X)
   {
   arma_extra_debug_sigprint();
   
-  access::rw(Mat<eT>::vec_state) = 1;
+  Mat<eT>::operator=(X);
+  
+  return *this;
+  }
+
+
+
+template<typename eT>
+template<typename T1>
+inline
+Col<eT>::Col(const Base<eT,T1>& X)
+  : Mat<eT>(arma_vec_indicator(), 1)
+  {
+  arma_extra_debug_sigprint();
   
   Mat<eT>::operator=(X.get_ref());
   }
@@ -328,24 +486,58 @@ Col<eT>::operator<<(const eT val)
 
 template<typename eT>
 arma_inline
-eT&
-Col<eT>::row(const uword row_num)
+const Op<Col<eT>,op_htrans>
+Col<eT>::t() const
   {
-  arma_debug_check( (row_num >= Mat<eT>::n_rows), "Col::row(): out of bounds" );
-  
-  return access::rw(Mat<eT>::mem[row_num]);
+  return Op<Col<eT>,op_htrans>(*this);
   }
 
 
 
 template<typename eT>
 arma_inline
-eT
-Col<eT>::row(const uword row_num) const
+const Op<Col<eT>,op_htrans>
+Col<eT>::ht() const
   {
-  arma_debug_check( (row_num >= Mat<eT>::n_rows), "Col::row(): out of bounds" );
+  return Op<Col<eT>,op_htrans>(*this);
+  }
+
+
+
+template<typename eT>
+arma_inline
+const Op<Col<eT>,op_strans>
+Col<eT>::st() const
+  {
+  return Op<Col<eT>,op_strans>(*this);
+  }
+
+
+
+template<typename eT>
+arma_inline
+subview_col<eT>
+Col<eT>::row(const uword in_row1)
+  {
+  arma_extra_debug_sigprint();
   
-  return Mat<eT>::mem[row_num];
+  arma_debug_check( (in_row1 >= Mat<eT>::n_rows), "Col::row(): indices out of bounds or incorrectly used");
+  
+  return subview_col<eT>(*this, 0, in_row1, 1);
+  }
+
+
+
+template<typename eT>
+arma_inline
+const subview_col<eT>
+Col<eT>::row(const uword in_row1) const
+  {
+  arma_extra_debug_sigprint();
+  
+  arma_debug_check( (in_row1 >= Mat<eT>::n_rows), "Col::row(): indices out of bounds or incorrectly used");
+  
+  return subview_col<eT>(*this, 0, in_row1, 1);
   }
 
 
@@ -414,6 +606,30 @@ Col<eT>::subvec(const uword in_row1, const uword in_row2) const
 
 
 
+template<typename eT>
+arma_inline
+subview_col<eT>
+Col<eT>::rows(const span& row_span)
+  {
+  arma_extra_debug_sigprint();
+  
+  return subvec(row_span);
+  }
+
+
+
+template<typename eT>
+arma_inline
+const subview_col<eT>
+Col<eT>::rows(const span& row_span) const
+  {
+  arma_extra_debug_sigprint();
+  
+  return subvec(row_span);
+  }
+
+
+
 template<typename eT>
 arma_inline
 subview_col<eT>
@@ -458,27 +674,27 @@ Col<eT>::subvec(const span& row_span) const
 
 
 
-// template<typename eT>
-// arma_inline
-// subview_col<eT>
-// Col<eT>::operator()(const span& row_span)
-//   {
-//   arma_extra_debug_sigprint();
-//   
-//   return subvec(row_span);
-//   }
-// 
-// 
-// 
-// template<typename eT>
-// arma_inline
-// const subview_col<eT>
-// Col<eT>::operator()(const span& row_span) const
-//   {
-//   arma_extra_debug_sigprint();
-//   
-//   return subvec(row_span);
-//   }
+template<typename eT>
+arma_inline
+subview_col<eT>
+Col<eT>::operator()(const span& row_span)
+  {
+  arma_extra_debug_sigprint();
+  
+  return subvec(row_span);
+  }
+
+
+
+template<typename eT>
+arma_inline
+const subview_col<eT>
+Col<eT>::operator()(const span& row_span) const
+  {
+  arma_extra_debug_sigprint();
+  
+  return subvec(row_span);
+  }
 
 
 
@@ -490,7 +706,7 @@ Col<eT>::shed_row(const uword row_num)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( row_num >= Mat<eT>::n_rows, "Col::shed_row(): out of bounds");
+  arma_debug_check( row_num >= Mat<eT>::n_rows, "Col::shed_row(): index out of bounds");
   
   shed_rows(row_num, row_num);
   }
@@ -549,7 +765,7 @@ Col<eT>::insert_rows(const uword row_num, const uword N, const bool set_to_zero)
   const uword B_n_rows = t_n_rows - row_num;
   
   // insertion at row_num == n_rows is in effect an append operation
-  arma_debug_check( (row_num > t_n_rows), "Col::insert_rows(): out of bounds");
+  arma_debug_check( (row_num > t_n_rows), "Col::insert_rows(): index out of bounds");
   
   if(N > 0)
     {
@@ -594,6 +810,50 @@ Col<eT>::insert_rows(const uword row_num, const Base<eT,T1>& X)
 
 
 
+template<typename eT>
+arma_inline
+arma_warn_unused
+eT&
+Col<eT>::at(const uword i)
+  {
+  return access::rw(Mat<eT>::mem[i]);
+  }
+
+
+
+template<typename eT>
+arma_inline
+arma_warn_unused
+const eT&
+Col<eT>::at(const uword i) const
+  {
+  return Mat<eT>::mem[i];
+  }
+
+
+
+template<typename eT>
+arma_inline
+arma_warn_unused
+eT&
+Col<eT>::at(const uword in_row, const uword)
+  {
+  return access::rw( Mat<eT>::mem[in_row] );
+  }
+
+
+
+template<typename eT>
+arma_inline
+arma_warn_unused
+const eT&
+Col<eT>::at(const uword in_row, const uword) const
+  {
+  return Mat<eT>::mem[in_row];
+  }
+
+
+
 template<typename eT>
 inline
 typename Col<eT>::row_iterator
@@ -650,24 +910,6 @@ Col<eT>::end_row(const uword row_num) const
 
 
 
-template<typename eT>
-template<uword fixed_n_elem>
-arma_inline
-void
-Col<eT>::fixed<fixed_n_elem>::mem_setup()
-  {
-  arma_extra_debug_sigprint();
-  
-  access::rw(Mat<eT>::n_rows)    = fixed_n_elem;
-  access::rw(Mat<eT>::n_cols)    = 1;
-  access::rw(Mat<eT>::n_elem)    = fixed_n_elem;
-  access::rw(Mat<eT>::vec_state) = 1;
-  access::rw(Mat<eT>::mem_state) = 3;
-  access::rw(Mat<eT>::mem)       = (use_extra) ? mem_local_extra : Mat<eT>::mem_local;
-  }
-
-
-
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
@@ -686,10 +928,9 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 Col<eT>::fixed<fixed_n_elem>::fixed()
+  : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
   arma_extra_debug_sigprint_this(this);
-  
-  mem_setup();
   }
 
 
@@ -698,14 +939,14 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 Col<eT>::fixed<fixed_n_elem>::fixed(const fixed<fixed_n_elem>& X)
+  : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
   arma_extra_debug_sigprint_this(this);
   
-  mem_setup();
+        eT* dest = (use_extra) ?   mem_local_extra : Mat<eT>::mem_local;
+  const eT* src  = (use_extra) ? X.mem_local_extra :        X.mem_local;
   
-  eT* dest = (use_extra) ? mem_local_extra : Mat<eT>::mem_local;
-  
-  arrayops::copy( dest, X.mem, fixed_n_elem );
+  arrayops::copy( dest, src, fixed_n_elem );
   }
 
 
@@ -714,26 +955,42 @@ template<typename eT>
 template<uword fixed_n_elem>
 inline
 Col<eT>::fixed<fixed_n_elem>::fixed(const subview_cube<eT>& X)
+  : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
   arma_extra_debug_sigprint_this(this);
   
-  mem_setup();
-  
   Col<eT>::operator=(X);
   }
 
 
 
+template<typename eT>
+template<uword fixed_n_elem>
+template<typename fill_type>
+inline
+Col<eT>::fixed<fixed_n_elem>::fixed(const fill::fill_class<fill_type>&)
+  : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
+  {
+  arma_extra_debug_sigprint_this(this);
+  
+  if(is_same_type<fill_type, fill::fill_zeros>::yes)  (*this).zeros();
+  if(is_same_type<fill_type, fill::fill_ones >::yes)  (*this).ones();
+  if(is_same_type<fill_type, fill::fill_eye  >::yes)  (*this).eye();
+  if(is_same_type<fill_type, fill::fill_randu>::yes)  (*this).randu();
+  if(is_same_type<fill_type, fill::fill_randn>::yes)  (*this).randn();
+  }
+
+
+
 template<typename eT>
 template<uword fixed_n_elem>
 template<typename T1>
 inline
 Col<eT>::fixed<fixed_n_elem>::fixed(const Base<eT,T1>& A)
+  : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
   arma_extra_debug_sigprint_this(this);
   
-  mem_setup();
-  
   Col<eT>::operator=(A.get_ref());
   }
 
@@ -744,55 +1001,26 @@ template<uword fixed_n_elem>
 template<typename T1, typename T2>
 inline
 Col<eT>::fixed<fixed_n_elem>::fixed(const Base<pod_type,T1>& A, const Base<pod_type,T2>& B)
+  : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
   arma_extra_debug_sigprint_this(this);
   
-  mem_setup();
-  
   Col<eT>::init(A,B);
   }
 
 
 
-template<typename eT>
-template<uword fixed_n_elem>
-inline
-Col<eT>::fixed<fixed_n_elem>::fixed(eT* aux_mem, const bool copy_aux_mem)
-  {
-  arma_extra_debug_sigprint_this(this);
-  
-  access::rw(Mat<eT>::n_rows)    = fixed_n_elem;
-  access::rw(Mat<eT>::n_cols)    = 1;
-  access::rw(Mat<eT>::n_elem)    = fixed_n_elem;
-  access::rw(Mat<eT>::vec_state) = 1;
-  access::rw(Mat<eT>::mem_state) = 3;
-  
-  if(copy_aux_mem == true)
-    {
-    eT* dest = (use_extra) ? mem_local_extra : Mat<eT>::mem_local;
-    
-    access::rw(Mat<eT>::mem) = dest;
-    
-    arrayops::copy( dest, aux_mem, fixed_n_elem );
-    }
-  else
-    {
-    access::rw(Mat<eT>::mem) = aux_mem;
-    }
-  }
-
-
-
 template<typename eT>
 template<uword fixed_n_elem>
 inline
 Col<eT>::fixed<fixed_n_elem>::fixed(const eT* aux_mem)
+  : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
   arma_extra_debug_sigprint_this(this);
   
-  mem_setup();
+  eT* dest = (use_extra) ? mem_local_extra : Mat<eT>::mem_local;
   
-  arrayops::copy( const_cast<eT*>(Mat<eT>::mem), aux_mem, fixed_n_elem );
+  arrayops::copy( dest, aux_mem, fixed_n_elem );
   }
 
 
@@ -804,11 +1032,10 @@ template<typename eT>
 template<uword fixed_n_elem>
 inline
 Col<eT>::fixed<fixed_n_elem>::fixed(const char* text)
+  : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
   arma_extra_debug_sigprint_this(this);
   
-  mem_setup();
-  
   change_to_row();
   
   Col<eT>::operator=(text);
@@ -823,11 +1050,10 @@ template<typename eT>
 template<uword fixed_n_elem>
 inline
 Col<eT>::fixed<fixed_n_elem>::fixed(const std::string& text)
+  : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
   arma_extra_debug_sigprint_this(this);
   
-  mem_setup();
-  
   change_to_row();
   
   Col<eT>::operator=(text);
@@ -910,80 +1136,115 @@ Col<eT>::fixed<fixed_n_elem>::operator=(const subview_cube<eT>& X)
 
 
 
+#if defined(ARMA_USE_CXX11)
+  
+  template<typename eT>
+  template<uword fixed_n_elem>
+  inline
+  Col<eT>::fixed<fixed_n_elem>::fixed(const std::initializer_list<eT>& list)
+    : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
+    {
+    arma_extra_debug_sigprint_this(this);
+    
+    (*this).operator=(list);
+    }
+  
+  
+  
+  template<typename eT>
+  template<uword fixed_n_elem>
+  inline
+  const Col<eT>&
+  Col<eT>::fixed<fixed_n_elem>::operator=(const std::initializer_list<eT>& list)
+    {
+    arma_extra_debug_sigprint();
+    
+    const uword N = list.size();
+    
+    arma_debug_check( (N > fixed_n_elem), "Col::fixed: initialiser list is too long" );
+    
+    eT* this_mem = (*this).memptr();
+    
+    arrayops::copy( this_mem, list.begin(), N );
+    
+    for(uword iq=N; iq < fixed_n_elem; ++iq) { this_mem[iq] = eT(0); }
+    
+    return *this;
+    }
+  
+#endif
+
+
+
 template<typename eT>
 template<uword fixed_n_elem>
-inline
-subview_row<eT>
-Col<eT>::fixed<fixed_n_elem>::operator()(const uword row_num, const span& col_span)
+arma_inline
+const Col<eT>&
+Col<eT>::fixed<fixed_n_elem>::operator=(const fixed<fixed_n_elem>& X)
   {
   arma_extra_debug_sigprint();
   
-  return Mat<eT>::operator()(row_num, col_span);
+        eT* dest = (use_extra) ?   mem_local_extra : Mat<eT>::mem_local;
+  const eT* src  = (use_extra) ? X.mem_local_extra :        X.mem_local;
+  
+  arrayops::copy( dest, src, fixed_n_elem );
+  
+  return *this;
   }
 
 
 
 template<typename eT>
 template<uword fixed_n_elem>
-inline
-const subview_row<eT>
-Col<eT>::fixed<fixed_n_elem>::operator()(const uword row_num, const span& col_span) const
+arma_inline
+const Op< typename Col<eT>::template fixed<fixed_n_elem>::Col_fixed_type, op_htrans >
+Col<eT>::fixed<fixed_n_elem>::t() const
   {
-  arma_extra_debug_sigprint();
-  
-  return Mat<eT>::operator()(row_num, col_span);
+  return Op< typename Col<eT>::template fixed<fixed_n_elem>::Col_fixed_type, op_htrans >(*this);
   }
 
 
 
 template<typename eT>
 template<uword fixed_n_elem>
-inline
-subview_col<eT>
-Col<eT>::fixed<fixed_n_elem>::operator()(const span& row_span, const uword col_num)
+arma_inline
+const Op< typename Col<eT>::template fixed<fixed_n_elem>::Col_fixed_type, op_htrans >
+Col<eT>::fixed<fixed_n_elem>::ht() const
   {
-  arma_extra_debug_sigprint();
-  
-  return Mat<eT>::operator()(row_span, col_num);
+  return Op< typename Col<eT>::template fixed<fixed_n_elem>::Col_fixed_type, op_htrans >(*this);
   }
 
 
 
 template<typename eT>
 template<uword fixed_n_elem>
-inline
-const subview_col<eT>
-Col<eT>::fixed<fixed_n_elem>::operator()(const span& row_span, const uword col_num) const
+arma_inline
+const Op< typename Col<eT>::template fixed<fixed_n_elem>::Col_fixed_type, op_strans >
+Col<eT>::fixed<fixed_n_elem>::st() const
   {
-  arma_extra_debug_sigprint();
-  
-  return Mat<eT>::operator()(row_span, col_num);
+  return Op< typename Col<eT>::template fixed<fixed_n_elem>::Col_fixed_type, op_strans >(*this);
   }
 
 
 
 template<typename eT>
 template<uword fixed_n_elem>
-inline
-subview<eT>
-Col<eT>::fixed<fixed_n_elem>::operator()(const span& row_span, const span& col_span)
+arma_inline
+arma_warn_unused
+const eT&
+Col<eT>::fixed<fixed_n_elem>::at_alt(const uword ii) const
   {
-  arma_extra_debug_sigprint();
+  #if defined(ARMA_HAVE_ALIGNED_ATTRIBUTE)
   
-  return Mat<eT>::operator()(row_span, col_span);
-  }
-
-
-
-template<typename eT>
-template<uword fixed_n_elem>
-inline
-const subview<eT>
-Col<eT>::fixed<fixed_n_elem>::operator()(const span& row_span, const span& col_span) const
-  {
-  arma_extra_debug_sigprint();
-  
-  return Mat<eT>::operator()(row_span, col_span);
+    return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
+    
+  #else
+    const eT* mem_aligned = (use_extra) ? mem_local_extra : Mat<eT>::mem_local;
+    
+    memory::mark_as_aligned(mem_aligned);
+    
+    return mem_aligned[ii];
+  #endif
   }
 
 
@@ -993,9 +1254,9 @@ template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
 eT&
-Col<eT>::fixed<fixed_n_elem>::operator[] (const uword i)
+Col<eT>::fixed<fixed_n_elem>::operator[] (const uword ii)
   {
-  return access::rw( Mat<eT>::mem[i] );
+  return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
   }
 
 
@@ -1004,10 +1265,10 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
-eT
-Col<eT>::fixed<fixed_n_elem>::operator[] (const uword i) const
+const eT&
+Col<eT>::fixed<fixed_n_elem>::operator[] (const uword ii) const
   {
-  return ( Mat<eT>::mem[i] );
+  return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
   }
 
 
@@ -1017,9 +1278,9 @@ template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
 eT&
-Col<eT>::fixed<fixed_n_elem>::at(const uword i)
+Col<eT>::fixed<fixed_n_elem>::at(const uword ii)
   {
-  return access::rw( Mat<eT>::mem[i] );
+  return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
   }
 
 
@@ -1028,10 +1289,10 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
-eT
-Col<eT>::fixed<fixed_n_elem>::at(const uword i) const
+const eT&
+Col<eT>::fixed<fixed_n_elem>::at(const uword ii) const
   {
-  return ( Mat<eT>::mem[i] );
+  return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
   }
 
 
@@ -1041,11 +1302,11 @@ template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
 eT&
-Col<eT>::fixed<fixed_n_elem>::operator() (const uword i)
+Col<eT>::fixed<fixed_n_elem>::operator() (const uword ii)
   {
-  arma_debug_check( (i >= fixed_n_elem), "Col::fixed::operator(): out of bounds");
+  arma_debug_check( (ii >= fixed_n_elem), "Col::operator(): index out of bounds");
   
-  return access::rw( Mat<eT>::mem[i] );
+  return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
   }
 
 
@@ -1054,12 +1315,12 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
-eT
-Col<eT>::fixed<fixed_n_elem>::operator() (const uword i) const
+const eT&
+Col<eT>::fixed<fixed_n_elem>::operator() (const uword ii) const
   {
-  arma_debug_check( (i >= fixed_n_elem), "Col::fixed::operator(): out of bounds");
+  arma_debug_check( (ii >= fixed_n_elem), "Col::operator(): index out of bounds");
   
-  return ( Mat<eT>::mem[i] );
+  return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
   }
 
 
@@ -1069,9 +1330,9 @@ template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
 eT&
-Col<eT>::fixed<fixed_n_elem>::at(const uword in_row, const uword in_col)
+Col<eT>::fixed<fixed_n_elem>::at(const uword in_row, const uword)
   {
-  return access::rw( Mat<eT>::mem[in_row] );
+  return (use_extra) ? mem_local_extra[in_row] : Mat<eT>::mem_local[in_row];
   }
 
 
@@ -1080,10 +1341,10 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
-eT
-Col<eT>::fixed<fixed_n_elem>::at(const uword in_row, const uword in_col) const
+const eT&
+Col<eT>::fixed<fixed_n_elem>::at(const uword in_row, const uword) const
   {
-  return ( Mat<eT>::mem[in_row] );
+  return (use_extra) ? mem_local_extra[in_row] : Mat<eT>::mem_local[in_row];
   }
 
 
@@ -1095,9 +1356,9 @@ arma_warn_unused
 eT&
 Col<eT>::fixed<fixed_n_elem>::operator() (const uword in_row, const uword in_col)
   {
-  arma_debug_check( ((in_row >= fixed_n_elem) || (in_col >= 1)), "Col::fixed::operator(): out of bounds" );
+  arma_debug_check( ((in_row >= fixed_n_elem) || (in_col > 0)), "Col::operator(): index out of bounds" );
   
-  return access::rw( Mat<eT>::mem[in_row] );
+  return (use_extra) ? mem_local_extra[in_row] : Mat<eT>::mem_local[in_row];
   }
 
 
@@ -1106,12 +1367,36 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
-eT
+const eT&
 Col<eT>::fixed<fixed_n_elem>::operator() (const uword in_row, const uword in_col) const
   {
-  arma_debug_check( ((in_row >= fixed_n_elem) || (in_col >= 1)), "Col::fixed::operator(): out of bounds" );
+  arma_debug_check( ((in_row >= fixed_n_elem) || (in_col > 0)), "Col::operator(): index out of bounds" );
   
-  return ( Mat<eT>::mem[in_row] );
+  return (use_extra) ? mem_local_extra[in_row] : Mat<eT>::mem_local[in_row];
+  }
+
+
+
+template<typename eT>
+template<uword fixed_n_elem>
+arma_inline
+arma_warn_unused
+eT*
+Col<eT>::fixed<fixed_n_elem>::memptr()
+  {
+  return (use_extra) ? mem_local_extra : Mat<eT>::mem_local;
+  }
+
+
+
+template<typename eT>
+template<uword fixed_n_elem>
+arma_inline
+arma_warn_unused
+const eT*
+Col<eT>::fixed<fixed_n_elem>::memptr() const
+  {
+  return (use_extra) ? mem_local_extra : Mat<eT>::mem_local;
   }
 
 
@@ -1125,7 +1410,9 @@ Col<eT>::fixed<fixed_n_elem>::fill(const eT val)
   {
   arma_extra_debug_sigprint();
   
-  arrayops::inplace_set( const_cast<eT*>(Mat<eT>::mem), val, fixed_n_elem );
+  eT* mem_use = (use_extra) ? &(mem_local_extra[0]) : &(Mat<eT>::mem_local[0]);
+  
+  arrayops::inplace_set_fixed<eT,fixed_n_elem>( mem_use, val );
   
   return *this;
   }
@@ -1141,7 +1428,9 @@ Col<eT>::fixed<fixed_n_elem>::zeros()
   {
   arma_extra_debug_sigprint();
   
-  arrayops::inplace_set( const_cast<eT*>(Mat<eT>::mem), eT(0), fixed_n_elem );
+  eT* mem_use = (use_extra) ? &(mem_local_extra[0]) : &(Mat<eT>::mem_local[0]);
+  
+  arrayops::inplace_set_fixed<eT,fixed_n_elem>( mem_use, eT(0) );
   
   return *this;
   }
@@ -1157,13 +1446,25 @@ Col<eT>::fixed<fixed_n_elem>::ones()
   {
   arma_extra_debug_sigprint();
   
-  arrayops::inplace_set( const_cast<eT*>(Mat<eT>::mem), eT(1), fixed_n_elem );
+  eT* mem_use = (use_extra) ? &(mem_local_extra[0]) : &(Mat<eT>::mem_local[0]);
+  
+  arrayops::inplace_set_fixed<eT,fixed_n_elem>( mem_use, eT(1) );
   
   return *this;
   }
 
 
 
+template<typename eT>
+inline
+Col<eT>::Col(const arma_fixed_indicator&, const uword in_n_elem, const eT* in_mem)
+  : Mat<eT>(arma_fixed_indicator(), in_n_elem, 1, 1, in_mem)
+  {
+  arma_extra_debug_sigprint_this(this);
+  }
+
+
+
 #ifdef ARMA_EXTRA_COL_MEAT
   #include ARMA_INCFILE_WRAP(ARMA_EXTRA_COL_MEAT)
 #endif
