@@ -1,14 +1,9 @@
-// Copyright (C) 2008-2012 NICTA (www.nicta.com.au)
-// Copyright (C) 2008-2012 Conrad Sanderson
+// Copyright (C) 2008-2013 Conrad Sanderson
+// Copyright (C) 2008-2013 NICTA (www.nicta.com.au)
 // 
-// This file is part of the Armadillo C++ library.
-// It is provided without any warranty of fitness
-// for any purpose. You can redistribute this file
-// and/or modify it under the terms of the GNU
-// Lesser General Public License (LGPL) as published
-// by the Free Software Foundation, either version 3
-// of the License or (at your option) any later version.
-// (see http://www.opensource.org/licenses for more info)
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 
 //! \addtogroup Row
@@ -61,6 +56,34 @@ Row<eT>::Row(const uword in_n_rows, const uword in_n_cols)
 
 
 
+template<typename eT>
+template<typename fill_type>
+inline
+Row<eT>::Row(const uword in_n_elem, const fill::fill_class<fill_type>& f)
+  : Mat<eT>(arma_vec_indicator(), 1, in_n_elem, 2)
+  {
+  arma_extra_debug_sigprint();
+  
+  (*this).fill(f);
+  }
+
+
+
+template<typename eT>
+template<typename fill_type>
+inline
+Row<eT>::Row(const uword in_n_rows, const uword in_n_cols, const fill::fill_class<fill_type>& f)
+  : Mat<eT>(arma_vec_indicator(), 2)
+  {
+  arma_extra_debug_sigprint();
+  
+  Mat<eT>::init_warm(in_n_rows, in_n_cols);
+  
+  (*this).fill(f);
+  }
+
+
+
 template<typename eT>
 inline
 Row<eT>::Row(const char* text)
@@ -115,37 +138,152 @@ Row<eT>::operator=(const std::string& text)
 
 
 
-#if defined(ARMA_USE_CXX11)
-
+//! create a row vector from std::vector
 template<typename eT>
 inline
-Row<eT>::Row(const std::initializer_list<eT>& list)
+Row<eT>::Row(const std::vector<eT>& x)
+  : Mat<eT>(arma_vec_indicator(), 1, uword(x.size()), 2)
   {
-  arma_extra_debug_sigprint();
+  arma_extra_debug_sigprint_this(this);
   
-  access::rw(Mat<eT>::vec_state) = 2;
-  
-  Mat<eT>::operator=(list);
+  if(x.size() > 0)
+    {
+    arrayops::copy( Mat<eT>::memptr(), &(x[0]), uword(x.size()) );
+    }
   }
-
-
-
+  
+  
+  
+//! create a row vector from std::vector
 template<typename eT>
 inline
 const Row<eT>&
-Row<eT>::operator=(const std::initializer_list<eT>& list)
+Row<eT>::operator=(const std::vector<eT>& x)
   {
   arma_extra_debug_sigprint();
   
-  Mat<eT>::operator=(list);
+  Mat<eT>::init_warm(1, uword(x.size()));
+  
+  if(x.size() > 0)
+    {
+    arrayops::copy( Mat<eT>::memptr(), &(x[0]), uword(x.size()) );
+    }
   
   return *this;
   }
 
+
+
+#if defined(ARMA_USE_CXX11)
+  
+  template<typename eT>
+  inline
+  Row<eT>::Row(const std::initializer_list<eT>& list)
+    {
+    arma_extra_debug_sigprint();
+    
+    access::rw(Mat<eT>::vec_state) = 2;
+    
+    Mat<eT>::operator=(list);
+    }
+  
+  
+  
+  template<typename eT>
+  inline
+  const Row<eT>&
+  Row<eT>::operator=(const std::initializer_list<eT>& list)
+    {
+    arma_extra_debug_sigprint();
+    
+    Mat<eT>::operator=(list);
+    
+    return *this;
+    }
+  
+  
+  
+  template<typename eT>
+  inline
+  Row<eT>::Row(Row<eT>&& X)
+    : Mat<eT>(arma_vec_indicator(), 2)
+    {
+    arma_extra_debug_sigprint(arma_boost::format("this = %x   X = %x") % this % &X);
+    
+    access::rw(Mat<eT>::n_rows) = 1;
+    access::rw(Mat<eT>::n_cols) = X.n_cols;
+    access::rw(Mat<eT>::n_elem) = X.n_elem;
+    
+    if( ((X.mem_state == 0) && (X.n_elem > arma_config::mat_prealloc)) || (X.mem_state == 1) || (X.mem_state == 2) )
+      {
+      access::rw(Mat<eT>::mem_state) = X.mem_state;
+      access::rw(Mat<eT>::mem)       = X.mem;
+      
+      access::rw(X.n_rows)    = 1;
+      access::rw(X.n_cols)    = 0;
+      access::rw(X.n_elem)    = 0;
+      access::rw(X.mem_state) = 0;
+      access::rw(X.mem)       = 0;
+      }
+    else
+      {
+      (*this).init_cold();
+      
+      arrayops::copy( (*this).memptr(), X.mem, X.n_elem );
+      
+      if( (X.mem_state == 0) && (X.n_elem <= arma_config::mat_prealloc) )
+        {
+        access::rw(X.n_rows) = 1;
+        access::rw(X.n_cols) = 0;
+        access::rw(X.n_elem) = 0;
+        access::rw(X.mem)    = 0;
+        }
+      }
+    }
+  
+  
+  
+  template<typename eT>
+  inline
+  const Row<eT>&
+  Row<eT>::operator=(Row<eT>&& X)
+    {
+    arma_extra_debug_sigprint(arma_boost::format("this = %x   X = %x") % this % &X);
+    
+    (*this).steal_mem(X);
+    
+    if( (X.mem_state == 0) && (X.n_elem <= arma_config::mat_prealloc) && (this != &X) )
+      {
+      access::rw(X.n_rows) = 1;
+      access::rw(X.n_cols) = 0;
+      access::rw(X.n_elem) = 0;
+      access::rw(X.mem)    = 0;
+      }
+    
+    return *this;
+    }
+  
 #endif
 
 
 
+template<typename eT>
+inline
+Row<eT>::Row(const SpRow<eT>& X)
+  : Mat<eT>(arma_vec_indicator(), 1, X.n_elem, 1)
+  {
+  arma_extra_debug_sigprint_this(this);
+
+  arrayops::inplace_set(Mat<eT>::memptr(), eT(0), X.n_elem);
+
+  for(typename SpRow<eT>::const_iterator it = X.begin(); it != X.end(); ++it)
+    {
+    at(it.col()) = (*it);
+    }
+  }
+
+
+
 template<typename eT>
 inline
 const Row<eT>&
@@ -160,6 +298,20 @@ Row<eT>::operator=(const eT val)
 
 
 
+template<typename eT>
+inline
+const Row<eT>&
+Row<eT>::operator=(const Row<eT>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  Mat<eT>::operator=(X);
+  
+  return *this;
+  }
+
+
+
 template<typename eT>
 template<typename T1>
 inline
@@ -330,24 +482,28 @@ Row<eT>::st() const
 
 template<typename eT>
 arma_inline
-eT&
-Row<eT>::col(const uword col_num)
+subview_row<eT>
+Row<eT>::col(const uword in_col1)
   {
-  arma_debug_check( (col_num >= Mat<eT>::n_cols), "Row::col(): out of bounds" );
+  arma_extra_debug_sigprint();
   
-  return access::rw(Mat<eT>::mem[col_num]);
+  arma_debug_check( (in_col1 >= Mat<eT>::n_cols), "Row::col(): indices out of bounds or incorrectly used");
+  
+  return subview_row<eT>(*this, 0, in_col1, 1);
   }
 
 
 
 template<typename eT>
 arma_inline
-eT
-Row<eT>::col(const uword col_num) const
+const subview_row<eT>
+Row<eT>::col(const uword in_col1) const
   {
-  arma_debug_check( (col_num >= Mat<eT>::n_cols), "Row::col(): out of bounds" );
+  arma_extra_debug_sigprint();
   
-  return Mat<eT>::mem[col_num];
+  arma_debug_check( (in_col1 >= Mat<eT>::n_cols), "Row::col(): indices out of bounds or incorrectly used");
+  
+  return subview_row<eT>(*this, 0, in_col1, 1);
   }
 
 
@@ -416,6 +572,30 @@ Row<eT>::subvec(const uword in_col1, const uword in_col2) const
 
 
 
+template<typename eT>
+arma_inline
+subview_row<eT>
+Row<eT>::cols(const span& col_span)
+  {
+  arma_extra_debug_sigprint();
+  
+  return subvec(col_span);
+  }
+
+
+
+template<typename eT>
+arma_inline
+const subview_row<eT>
+Row<eT>::cols(const span& col_span) const
+  {
+  arma_extra_debug_sigprint();
+  
+  return subvec(col_span);
+  }
+
+
+
 template<typename eT>
 arma_inline
 subview_row<eT>
@@ -492,7 +672,7 @@ Row<eT>::shed_col(const uword col_num)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( col_num >= Mat<eT>::n_cols, "Row::shed_col(): out of bounds");
+  arma_debug_check( col_num >= Mat<eT>::n_cols, "Row::shed_col(): index out of bounds");
   
   shed_cols(col_num, col_num);
   }
@@ -551,7 +731,7 @@ Row<eT>::insert_cols(const uword col_num, const uword N, const bool set_to_zero)
   const uword B_n_cols = t_n_cols - col_num;
   
   // insertion at col_num == n_cols is in effect an append operation
-  arma_debug_check( (col_num > t_n_cols), "Row::insert_cols(): out of bounds");
+  arma_debug_check( (col_num > t_n_cols), "Row::insert_cols(): index out of bounds");
   
   if(N > 0)
     {
@@ -610,7 +790,7 @@ Row<eT>::at(const uword i)
 template<typename eT>
 arma_inline
 arma_warn_unused
-eT
+const eT&
 Row<eT>::at(const uword i) const
   {
   return Mat<eT>::mem[i];
@@ -632,7 +812,7 @@ Row<eT>::at(const uword, const uword in_col)
 template<typename eT>
 arma_inline
 arma_warn_unused
-eT
+const eT&
 Row<eT>::at(const uword, const uword in_col) const
   {
   return Mat<eT>::mem[in_col];
@@ -715,9 +895,10 @@ Row<eT>::fixed<fixed_n_elem>::fixed(const fixed<fixed_n_elem>& X)
   {
   arma_extra_debug_sigprint_this(this);
   
-  eT* dest = (use_extra) ? mem_local_extra : Mat<eT>::mem_local;
+        eT* dest = (use_extra) ?   mem_local_extra : Mat<eT>::mem_local;
+  const eT* src  = (use_extra) ? X.mem_local_extra :        X.mem_local;
   
-  arrayops::copy( dest, X.mem, fixed_n_elem );
+  arrayops::copy( dest, src, fixed_n_elem );
   }
 
 
@@ -735,6 +916,24 @@ Row<eT>::fixed<fixed_n_elem>::fixed(const subview_cube<eT>& X)
 
 
 
+template<typename eT>
+template<uword fixed_n_elem>
+template<typename fill_type>
+inline
+Row<eT>::fixed<fixed_n_elem>::fixed(const fill::fill_class<fill_type>&)
+  : Row<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
+  {
+  arma_extra_debug_sigprint_this(this);
+  
+  if(is_same_type<fill_type, fill::fill_zeros>::yes)  (*this).zeros();
+  if(is_same_type<fill_type, fill::fill_ones >::yes)  (*this).ones();
+  if(is_same_type<fill_type, fill::fill_eye  >::yes)  (*this).eye();
+  if(is_same_type<fill_type, fill::fill_randu>::yes)  (*this).randu();
+  if(is_same_type<fill_type, fill::fill_randn>::yes)  (*this).randn();
+  }
+
+
+
 template<typename eT>
 template<uword fixed_n_elem>
 template<typename T1>
@@ -876,42 +1075,115 @@ Row<eT>::fixed<fixed_n_elem>::operator=(const subview_cube<eT>& X)
 
 
 #if defined(ARMA_USE_CXX11)
-
-template<typename eT>
-template<uword fixed_n_elem>
-inline
-Row<eT>::fixed<fixed_n_elem>::fixed(const std::initializer_list<eT>& list)
-  : Row<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
-  {
-  arma_extra_debug_sigprint_this(this);
   
-  (*this).operator=(list);
-  }
+  template<typename eT>
+  template<uword fixed_n_elem>
+  inline
+  Row<eT>::fixed<fixed_n_elem>::fixed(const std::initializer_list<eT>& list)
+    : Row<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
+    {
+    arma_extra_debug_sigprint_this(this);
+    
+    (*this).operator=(list);
+    }
+  
+  
+  
+  template<typename eT>
+  template<uword fixed_n_elem>
+  inline
+  const Row<eT>&
+  Row<eT>::fixed<fixed_n_elem>::operator=(const std::initializer_list<eT>& list)
+    {
+    arma_extra_debug_sigprint();
+    
+    const uword N = list.size();
+    
+    arma_debug_check( (N > fixed_n_elem), "Row::fixed: initialiser list is too long" );
+    
+    eT* this_mem = (*this).memptr();
+    
+    arrayops::copy( this_mem, list.begin(), N );
+    
+    for(uword iq=N; iq < fixed_n_elem; ++iq) { this_mem[iq] = eT(0); }
+    
+    return *this;
+    }
+  
+#endif
 
 
 
 template<typename eT>
 template<uword fixed_n_elem>
-inline
+arma_inline
 const Row<eT>&
-Row<eT>::fixed<fixed_n_elem>::operator=(const std::initializer_list<eT>& list)
+Row<eT>::fixed<fixed_n_elem>::operator=(const fixed<fixed_n_elem>& X)
   {
   arma_extra_debug_sigprint();
   
-  const uword N = list.size();
+        eT* dest = (use_extra) ?   mem_local_extra : Mat<eT>::mem_local;
+  const eT* src  = (use_extra) ? X.mem_local_extra :        X.mem_local;
   
-  arma_debug_check( (N > fixed_n_elem), "Row::fixed: initialiser list is too long" );
-  
-  eT* this_mem = (*this).memptr();
-  
-  arrayops::copy( this_mem, list.begin(), N );
-  
-  for(uword iq=N; iq < fixed_n_elem; ++iq) { this_mem[iq] = eT(0); }
+  arrayops::copy( dest, src, fixed_n_elem );
   
   return *this;
   }
 
-#endif
+
+
+template<typename eT>
+template<uword fixed_n_elem>
+arma_inline
+const Op< typename Row<eT>::template fixed<fixed_n_elem>::Row_fixed_type, op_htrans >
+Row<eT>::fixed<fixed_n_elem>::t() const
+  {
+  return Op< typename Row<eT>::template fixed<fixed_n_elem>::Row_fixed_type, op_htrans >(*this);
+  }
+
+
+
+template<typename eT>
+template<uword fixed_n_elem>
+arma_inline
+const Op< typename Row<eT>::template fixed<fixed_n_elem>::Row_fixed_type, op_htrans >
+Row<eT>::fixed<fixed_n_elem>::ht() const
+  {
+  return Op< typename Row<eT>::template fixed<fixed_n_elem>::Row_fixed_type, op_htrans >(*this);
+  }
+
+
+
+template<typename eT>
+template<uword fixed_n_elem>
+arma_inline
+const Op< typename Row<eT>::template fixed<fixed_n_elem>::Row_fixed_type, op_strans >
+Row<eT>::fixed<fixed_n_elem>::st() const
+  {
+  return Op< typename Row<eT>::template fixed<fixed_n_elem>::Row_fixed_type, op_strans >(*this);
+  }
+
+
+
+template<typename eT>
+template<uword fixed_n_elem>
+arma_inline
+arma_warn_unused
+const eT&
+Row<eT>::fixed<fixed_n_elem>::at_alt(const uword ii) const
+  {
+  #if defined(ARMA_HAVE_ALIGNED_ATTRIBUTE)
+  
+    return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
+    
+  #else
+    const eT* mem_aligned = (use_extra) ? mem_local_extra : Mat<eT>::mem_local;
+    
+    memory::mark_as_aligned(mem_aligned);
+    
+    return mem_aligned[ii];
+  #endif
+  }
 
 
 
@@ -931,7 +1203,7 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
-eT
+const eT&
 Row<eT>::fixed<fixed_n_elem>::operator[] (const uword ii) const
   {
   return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
@@ -955,7 +1227,7 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
-eT
+const eT&
 Row<eT>::fixed<fixed_n_elem>::at(const uword ii) const
   {
   return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
@@ -970,7 +1242,7 @@ arma_warn_unused
 eT&
 Row<eT>::fixed<fixed_n_elem>::operator() (const uword ii)
   {
-  arma_debug_check( (ii >= fixed_n_elem), "Row::operator(): out of bounds");
+  arma_debug_check( (ii >= fixed_n_elem), "Row::operator(): index out of bounds");
   
   return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
   }
@@ -981,10 +1253,10 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
-eT
+const eT&
 Row<eT>::fixed<fixed_n_elem>::operator() (const uword ii) const
   {
-  arma_debug_check( (ii >= fixed_n_elem), "Row::operator(): out of bounds");
+  arma_debug_check( (ii >= fixed_n_elem), "Row::operator(): index out of bounds");
   
   return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
   }
@@ -1007,7 +1279,7 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
-eT
+const eT&
 Row<eT>::fixed<fixed_n_elem>::at(const uword, const uword in_col) const
   {
   return (use_extra) ? mem_local_extra[in_col] : Mat<eT>::mem_local[in_col];
@@ -1022,7 +1294,7 @@ arma_warn_unused
 eT&
 Row<eT>::fixed<fixed_n_elem>::operator() (const uword in_row, const uword in_col)
   {
-  arma_debug_check( ((in_row > 0) || (in_col >= fixed_n_elem)), "Row::operator(): out of bounds" );
+  arma_debug_check( ((in_row > 0) || (in_col >= fixed_n_elem)), "Row::operator(): index out of bounds" );
   
   return (use_extra) ? mem_local_extra[in_col] : Mat<eT>::mem_local[in_col];
   }
@@ -1033,10 +1305,10 @@ template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
 arma_warn_unused
-eT
+const eT&
 Row<eT>::fixed<fixed_n_elem>::operator() (const uword in_row, const uword in_col) const
   {
-  arma_debug_check( ((in_row > 0) || (in_col >= fixed_n_elem)), "Row::operator(): out of bounds" );
+  arma_debug_check( ((in_row > 0) || (in_col >= fixed_n_elem)), "Row::operator(): index out of bounds" );
   
   return (use_extra) ? mem_local_extra[in_col] : Mat<eT>::mem_local[in_col];
   }
