@@ -1,15 +1,10 @@
-// Copyright (C) 2008-2011 NICTA (www.nicta.com.au)
-// Copyright (C) 2008-2011 Conrad Sanderson
+// Copyright (C) 2008-2013 Conrad Sanderson
+// Copyright (C) 2008-2013 NICTA (www.nicta.com.au)
 // Copyright (C) 2011 Stanislav Funiak
 // 
-// This file is part of the Armadillo C++ library.
-// It is provided without any warranty of fitness
-// for any purpose. You can redistribute this file
-// and/or modify it under the terms of the GNU
-// Lesser General Public License (LGPL) as published
-// by the Free Software Foundation, either version 3
-// of the License or (at your option) any later version.
-// (see http://www.opensource.org/licenses for more info)
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 
 //! \addtogroup debug
@@ -92,12 +87,13 @@ get_stream_err2()
 
 //! print a message to get_stream_err1() and/or throw a logic_error exception
 template<typename T1>
-inline
-void
 arma_cold
+arma_noinline
+static
+void
 arma_stop(const T1& x)
   {
-  #if defined(ARMA_PRINT_LOGIC_ERRORS)
+  #if defined(ARMA_PRINT_ERRORS)
     {
     std::ostream& out = get_stream_err1();
     
@@ -114,25 +110,34 @@ arma_stop(const T1& x)
     }
   #endif
   
-  throw std::logic_error("");
+  throw std::logic_error( std::string(x) );
   }
 
 
 
 template<typename T1>
-inline
-void
 arma_cold
+arma_noinline
+static
+void
 arma_stop_bad_alloc(const T1& x)
   {
-  std::ostream& out = get_stream_err1();
-  
-  out.flush();
-  
-  out << '\n';
-  out << "error: " << x << '\n';
-  out << '\n';
-  out.flush();
+  #if defined(ARMA_PRINT_ERRORS)
+    {
+    std::ostream& out = get_stream_err2();
+    
+    out.flush();
+    
+    out << '\n';
+    out << "error: " << x << '\n';
+    out << '\n';
+    out.flush();
+    }
+  #else
+    {
+    arma_ignore(x);
+    }
+  #endif
   
   throw std::bad_alloc();
   }
@@ -144,12 +149,13 @@ arma_stop_bad_alloc(const T1& x)
 
 //! print a message to get_stream_err2() and/or throw a run-time error exception
 template<typename T1>
-inline
-void
 arma_cold
+arma_noinline
+static
+void
 arma_bad(const T1& x, const bool hurl = true)
   {
-  #if defined(ARMA_PRINT_RUNTIME_ERRORS)
+  #if defined(ARMA_PRINT_ERRORS)
     {
     std::ostream& out = get_stream_err2();
     
@@ -168,7 +174,7 @@ arma_bad(const T1& x, const bool hurl = true)
   
   if(hurl == true)
     {
-    throw std::runtime_error("");
+    throw std::runtime_error( std::string(x) );
     }
   }
 
@@ -178,9 +184,9 @@ arma_bad(const T1& x, const bool hurl = true)
 // arma_print
 
 
+arma_cold
 inline
 void
-arma_cold
 arma_print()
   {
   get_stream_err1() << std::endl;
@@ -188,9 +194,10 @@ arma_print()
 
 
 template<typename T1>
-inline
-void
 arma_cold
+arma_noinline
+static
+void
 arma_print(const T1& x)
   {
   get_stream_err1() << x << std::endl;
@@ -199,9 +206,10 @@ arma_print(const T1& x)
 
 
 template<typename T1, typename T2>
-inline
-void
 arma_cold
+arma_noinline
+static
+void
 arma_print(const T1& x, const T2& y)
   {
   get_stream_err1() << x << y << std::endl;
@@ -210,9 +218,10 @@ arma_print(const T1& x, const T2& y)
 
 
 template<typename T1, typename T2, typename T3>
-inline
-void
 arma_cold
+arma_noinline
+static
+void
 arma_print(const T1& x, const T2& y, const T3& z)
   {
   get_stream_err1() << x << y << z << std::endl;
@@ -292,9 +301,10 @@ arma_thisprint(const void* this_ptr)
 
 //! print a message to the warn stream
 template<typename T1>
-inline
-void
 arma_cold
+arma_noinline
+static
+void
 arma_warn(const bool state, const T1& x)
   {
   if(state==true)
@@ -305,9 +315,10 @@ arma_warn(const bool state, const T1& x)
 
 
 template<typename T1, typename T2>
-inline
-void
 arma_cold
+arma_noinline
+static
+void
 arma_warn(const bool state, const T1& x, const T2& y)
   {
   if(state==true)
@@ -318,9 +329,10 @@ arma_warn(const bool state, const T1& x, const T2& y)
 
 
 template<typename T1, typename T2, typename T3>
-inline
-void
 arma_cold
+arma_noinline
+static
+void
 arma_warn(const bool state, const T1& x, const T2& y, const T3& z)
   {
   if(state==true)
@@ -336,9 +348,9 @@ arma_warn(const bool state, const T1& x, const T2& y, const T3& z)
 
 //! if state is true, abort program
 template<typename T1>
+arma_hot
 inline
 void
-arma_hot
 arma_check(const bool state, const T1& x)
   {
   if(state==true)
@@ -349,9 +361,9 @@ arma_check(const bool state, const T1& x)
 
 
 template<typename T1, typename T2>
+arma_hot
 inline
 void
-arma_hot
 arma_check(const bool state, const T1& x, const T2& y)
   {
   if(state==true)
@@ -362,9 +374,9 @@ arma_check(const bool state, const T1& x, const T2& y)
 
 
 template<typename T1>
+arma_hot
 inline
 void
-arma_hot
 arma_check_bad_alloc(const bool state, const T1& x)
   {
   if(state==true)
@@ -379,9 +391,9 @@ arma_check_bad_alloc(const bool state, const T1& x)
 // arma_set_error
 
 
+arma_hot
 arma_inline
 void
-arma_hot
 arma_set_error(bool& err_state, char*& err_msg, const bool expression, const char* message)
   {
   if(expression == true)
@@ -397,9 +409,10 @@ arma_set_error(bool& err_state, char*& err_msg, const bool expression, const cha
 //
 // functions for generating strings indicating size errors
 
-inline
-std::string
 arma_cold
+arma_noinline
+static
+std::string
 arma_incompat_size_string(const uword A_n_rows, const uword A_n_cols, const uword B_n_rows, const uword B_n_cols, const char* x)
   {
   std::stringstream tmp;
@@ -411,8 +424,9 @@ arma_incompat_size_string(const uword A_n_rows, const uword A_n_cols, const uwor
 
 
 
-inline
 arma_cold
+arma_noinline
+static
 std::string
 arma_incompat_size_string(const uword A_n_rows, const uword A_n_cols, const uword A_n_slices, const uword B_n_rows, const uword B_n_cols, const uword B_n_slices, const char* x)
   {
@@ -426,8 +440,9 @@ arma_incompat_size_string(const uword A_n_rows, const uword A_n_cols, const uwor
 
 
 template<typename eT>
-inline
 arma_cold
+arma_noinline
+static
 std::string
 arma_incompat_size_string(const subview_cube<eT>& Q, const Mat<eT>& A, const char* x)
   {
@@ -468,9 +483,9 @@ arma_assert_same_size(const uword A_n_rows, const uword A_n_cols, const uword B_
 
 //! stop if given matrices have different sizes
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const Mat<eT1>& A, const Mat<eT2>& B, const char* x)
   {
   const uword A_n_rows = A.n_rows;
@@ -489,9 +504,9 @@ arma_assert_same_size(const Mat<eT1>& A, const Mat<eT2>& B, const char* x)
 
 //! stop if given proxies have different sizes
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const Proxy<eT1>& A, const Proxy<eT2>& B, const char* x)
   {
   const uword A_n_rows = A.get_n_rows();
@@ -509,9 +524,9 @@ arma_assert_same_size(const Proxy<eT1>& A, const Proxy<eT2>& B, const char* x)
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const subview<eT1>& A, const subview<eT2>& B, const char* x)
   {
   const uword A_n_rows = A.n_rows;
@@ -529,9 +544,9 @@ arma_assert_same_size(const subview<eT1>& A, const subview<eT2>& B, const char*
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const Mat<eT1>& A, const subview<eT2>& B, const char* x)
   {
   const uword A_n_rows = A.n_rows;
@@ -549,9 +564,9 @@ arma_assert_same_size(const Mat<eT1>& A, const subview<eT2>& B, const char* x)
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const subview<eT1>& A, const Mat<eT2>& B, const char* x)
   {
   const uword A_n_rows = A.n_rows;
@@ -569,9 +584,9 @@ arma_assert_same_size(const subview<eT1>& A, const Mat<eT2>& B, const char* x)
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const Mat<eT1>& A, const Proxy<eT2>& B, const char* x)
   {
   const uword A_n_rows = A.n_rows;
@@ -589,9 +604,9 @@ arma_assert_same_size(const Mat<eT1>& A, const Proxy<eT2>& B, const char* x)
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const Proxy<eT1>& A, const Mat<eT2>& B, const char* x)
   {
   const uword A_n_rows = A.get_n_rows();
@@ -609,9 +624,9 @@ arma_assert_same_size(const Proxy<eT1>& A, const Mat<eT2>& B, const char* x)
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const Proxy<eT1>& A, const subview<eT2>& B, const char* x)
   {
   const uword A_n_rows = A.get_n_rows();
@@ -629,9 +644,9 @@ arma_assert_same_size(const Proxy<eT1>& A, const subview<eT2>& B, const char* x)
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const subview<eT1>& A, const Proxy<eT2>& B, const char* x)
   {
   const uword A_n_rows = A.n_rows;
@@ -653,9 +668,9 @@ arma_assert_same_size(const subview<eT1>& A, const Proxy<eT2>& B, const char* x)
 
 
 
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const uword A_n_rows, const uword A_n_cols, const uword A_n_slices, const uword B_n_rows, const uword B_n_cols, const uword B_n_slices, const char* x)
   {
   if( (A_n_rows != B_n_rows) || (A_n_cols != B_n_cols) || (A_n_slices != B_n_slices) )
@@ -668,9 +683,9 @@ arma_assert_same_size(const uword A_n_rows, const uword A_n_cols, const uword A_
 
 //! stop if given cubes have different sizes
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const Cube<eT1>& A, const Cube<eT2>& B, const char* x)
   {
   if( (A.n_rows != B.n_rows) || (A.n_cols != B.n_cols) || (A.n_slices != B.n_slices) )
@@ -682,9 +697,9 @@ arma_assert_same_size(const Cube<eT1>& A, const Cube<eT2>& B, const char* x)
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const Cube<eT1>& A, const subview_cube<eT2>& B, const char* x)
   {
   if( (A.n_rows != B.n_rows) || (A.n_cols != B.n_cols) || (A.n_slices != B.n_slices) )
@@ -696,9 +711,9 @@ arma_assert_same_size(const Cube<eT1>& A, const subview_cube<eT2>& B, const char
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const subview_cube<eT1>& A, const Cube<eT2>& B, const char* x)
   {
   if( (A.n_rows != B.n_rows) || (A.n_cols != B.n_cols) || (A.n_slices != B.n_slices) )
@@ -710,9 +725,9 @@ arma_assert_same_size(const subview_cube<eT1>& A, const Cube<eT2>& B, const char
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const subview_cube<eT1>& A, const subview_cube<eT2>& B, const char* x)
   {
   if( (A.n_rows != B.n_rows) || (A.n_cols != B.n_cols) || (A.n_slices != B.n_slices))
@@ -725,9 +740,9 @@ arma_assert_same_size(const subview_cube<eT1>& A, const subview_cube<eT2>& B, co
 
 //! stop if given cube proxies have different sizes
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const ProxyCube<eT1>& A, const ProxyCube<eT2>& B, const char* x)
   {
   const uword A_n_rows   = A.get_n_rows();
@@ -752,9 +767,9 @@ arma_assert_same_size(const ProxyCube<eT1>& A, const ProxyCube<eT2>& B, const ch
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const Cube<eT1>& A, const Mat<eT2>& B, const char* x)
   {
   if( (A.n_rows != B.n_rows) || (A.n_cols != B.n_cols) || (A.n_slices != 1) )
@@ -766,9 +781,9 @@ arma_assert_same_size(const Cube<eT1>& A, const Mat<eT2>& B, const char* x)
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const Mat<eT1>& A, const Cube<eT2>& B, const char* x)
   {
   if( (A.n_rows != B.n_rows) || (A.n_cols != B.n_cols) || (1 != B.n_slices) )
@@ -780,9 +795,9 @@ arma_assert_same_size(const Mat<eT1>& A, const Cube<eT2>& B, const char* x)
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const subview_cube<eT1>& A, const Mat<eT2>& B, const char* x)
   {
   if( (A.n_rows != B.n_rows) || (A.n_cols != B.n_cols) || (A.n_slices != 1) )
@@ -794,9 +809,9 @@ arma_assert_same_size(const subview_cube<eT1>& A, const Mat<eT2>& B, const char*
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_same_size(const Mat<eT1>& A, const subview_cube<eT2>& B, const char* x)
   {
   if( (A.n_rows != B.n_rows) || (A.n_cols != B.n_cols) || (1 != B.n_slices) )
@@ -962,9 +977,9 @@ arma_assert_cube_as_mat(const Mat<eT>& M, const T1& Q, const char* x, const bool
 
 
 
+arma_hot
 inline
 void
-arma_hot
 arma_assert_mul_size(const uword A_n_rows, const uword A_n_cols, const uword B_n_rows, const uword B_n_cols, const char* x)
   {
   if(A_n_cols != B_n_rows)
@@ -977,9 +992,9 @@ arma_assert_mul_size(const uword A_n_rows, const uword A_n_cols, const uword B_n
 
 //! stop if given matrices are incompatible for multiplication
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_mul_size(const Mat<eT1>& A, const Mat<eT2>& B, const char* x)
   {
   const uword A_n_cols = A.n_cols;
@@ -995,9 +1010,9 @@ arma_assert_mul_size(const Mat<eT1>& A, const Mat<eT2>& B, const char* x)
 
 //! stop if given matrices are incompatible for multiplication
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_mul_size(const Mat<eT1>& A, const Mat<eT2>& B, const bool do_trans_A, const bool do_trans_B, const char* x)
   {
   const uword final_A_n_cols = (do_trans_A == false) ? A.n_cols : A.n_rows;
@@ -1014,10 +1029,30 @@ arma_assert_mul_size(const Mat<eT1>& A, const Mat<eT2>& B, const bool do_trans_A
 
 
 
-template<typename eT1, typename eT2>
+template<const bool do_trans_A, const bool do_trans_B>
+arma_hot
 inline
 void
+arma_assert_trans_mul_size(const uword A_n_rows, const uword A_n_cols, const uword B_n_rows, const uword B_n_cols, const char* x)
+  {
+  const uword final_A_n_cols = (do_trans_A == false) ? A_n_cols : A_n_rows;
+  const uword final_B_n_rows = (do_trans_B == false) ? B_n_rows : B_n_cols;
+    
+  if(final_A_n_cols != final_B_n_rows)
+    {
+    const uword final_A_n_rows = (do_trans_A == false) ? A_n_rows : A_n_cols;
+    const uword final_B_n_cols = (do_trans_B == false) ? B_n_cols : B_n_rows;
+    
+    arma_stop( arma_incompat_size_string(final_A_n_rows, final_A_n_cols, final_B_n_rows, final_B_n_cols, x) );
+    }
+  }
+
+
+
+template<typename eT1, typename eT2>
 arma_hot
+inline
+void
 arma_assert_mul_size(const Mat<eT1>& A, const subview<eT2>& B, const char* x)
   {
   if(A.n_cols != B.n_rows)
@@ -1029,9 +1064,9 @@ arma_assert_mul_size(const Mat<eT1>& A, const subview<eT2>& B, const char* x)
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_mul_size(const subview<eT1>& A, const Mat<eT2>& B, const char* x)
   {
   if(A.n_cols != B.n_rows)
@@ -1043,9 +1078,9 @@ arma_assert_mul_size(const subview<eT1>& A, const Mat<eT2>& B, const char* x)
 
 
 template<typename eT1, typename eT2>
+arma_hot
 inline
 void
-arma_hot
 arma_assert_mul_size(const subview<eT1>& A, const subview<eT2>& B, const char* x)
   {
   if(A.n_cols != B.n_rows)
@@ -1060,47 +1095,35 @@ arma_assert_mul_size(const subview<eT1>& A, const subview<eT2>& B, const char* x
 // macros
 
 
-#define ARMA_STRING1(x) #x
-#define ARMA_STRING2(x) ARMA_STRING1(x)
-#define ARMA_FILELINE  __FILE__ ": " ARMA_STRING2(__LINE__)
+// #define ARMA_STRING1(x) #x
+// #define ARMA_STRING2(x) ARMA_STRING1(x)
+// #define ARMA_FILELINE  __FILE__ ": " ARMA_STRING2(__LINE__)
 
 
-#if defined (__GNUG__)
-  #define ARMA_FNSIG  __PRETTY_FUNCTION__
-#elif defined (_MSC_VER)
-  #define ARMA_FNSIG  __FUNCSIG__ 
-#elif defined (ARMA_USE_BOOST)
-  #define ARMA_FNSIG  BOOST_CURRENT_FUNCTION  
-#elif defined (ARMA_USE_CXX11)
-  #define ARMA_FNSIG  __func__
-#else 
-  #define ARMA_FNSIG  "(unknown)"
-#endif
-
-
-
-#if !defined(ARMA_NO_DEBUG) && !defined(NDEBUG)
-  
-  #define arma_debug_print              arma_print
-  #define arma_debug_warn               arma_warn
-  #define arma_debug_check              arma_check
-  #define arma_debug_set_error          arma_set_error
-  #define arma_debug_assert_same_size   arma_assert_same_size
-  #define arma_debug_assert_mul_size    arma_assert_mul_size
-  #define arma_debug_assert_cube_as_mat arma_assert_cube_as_mat
-  
-#else
+#if defined(ARMA_NO_DEBUG)
   
   #undef ARMA_EXTRA_DEBUG
   
-  #define arma_debug_print              true ? (void)0 : arma_print
-  #define arma_debug_warn               true ? (void)0 : arma_warn
-  #define arma_debug_check              true ? (void)0 : arma_check
-  #define arma_debug_set_error          true ? (void)0 : arma_set_error
-  #define arma_debug_assert_same_size   true ? (void)0 : arma_assert_same_size
-  #define arma_debug_assert_mul_size    true ? (void)0 : arma_assert_mul_size
-  #define arma_debug_assert_cube_as_mat true ? (void)0 : arma_assert_cube_as_mat
-
+  #define arma_debug_print                 true ? (void)0 : arma_print
+  #define arma_debug_warn                  true ? (void)0 : arma_warn
+  #define arma_debug_check                 true ? (void)0 : arma_check
+  #define arma_debug_set_error             true ? (void)0 : arma_set_error
+  #define arma_debug_assert_same_size      true ? (void)0 : arma_assert_same_size
+  #define arma_debug_assert_mul_size       true ? (void)0 : arma_assert_mul_size
+  #define arma_debug_assert_trans_mul_size true ? (void)0 : arma_assert_trans_mul_size
+  #define arma_debug_assert_cube_as_mat    true ? (void)0 : arma_assert_cube_as_mat
+  
+#else
+  
+  #define arma_debug_print                 arma_print
+  #define arma_debug_warn                  arma_warn
+  #define arma_debug_check                 arma_check
+  #define arma_debug_set_error             arma_set_error
+  #define arma_debug_assert_same_size      arma_assert_same_size
+  #define arma_debug_assert_mul_size       arma_assert_mul_size
+  #define arma_debug_assert_trans_mul_size arma_assert_trans_mul_size
+  #define arma_debug_assert_cube_as_mat    arma_assert_cube_as_mat
+  
 #endif
 
 
@@ -1135,7 +1158,6 @@ arma_assert_mul_size(const subview<eT1>& A, const subview<eT2>& B, const char* x
       public:
       
       inline
-      arma_cold
       arma_first_extra_debug_message()
         {
         union
@@ -1156,14 +1178,16 @@ arma_assert_mul_size(const subview<eT1>& A, const subview<eT2>& B, const char* x
             << arma_version::major << '.' << arma_version::minor << '.' << arma_version::patch
             << " (" << nickname << ")\n";
         
-        out << "@ arma_config::mat_prealloc   = " << arma_config::mat_prealloc   << " element(s)\n";
-        out << "@ arma_config::atlas          = " << arma_config::atlas          << '\n';
-        out << "@ arma_config::lapack         = " << arma_config::lapack         << '\n';
-        out << "@ arma_config::blas           = " << arma_config::blas           << '\n';
-        out << "@ arma_config::boost          = " << arma_config::boost          << '\n';
-        out << "@ arma_config::boost_date     = " << arma_config::boost_date     << '\n';
-        out << "@ arma_config::good_comp      = " << arma_config::good_comp      << '\n';
-        out << "@ arma_config::extra_code     = " << arma_config::extra_code     << '\n';
+        out << "@ arma_config::use_wrapper  = " << arma_config::use_wrapper  << '\n';
+        out << "@ arma_config::use_cxx11    = " << arma_config::use_cxx11    << '\n';
+        out << "@ arma_config::lapack       = " << arma_config::lapack       << '\n';
+        out << "@ arma_config::blas         = " << arma_config::blas         << '\n';
+        out << "@ arma_config::arpack       = " << arma_config::arpack       << '\n';
+        out << "@ arma_config::atlas        = " << arma_config::atlas        << '\n';
+        out << "@ arma_config::hdf5         = " << arma_config::hdf5         << '\n';
+        out << "@ arma_config::good_comp    = " << arma_config::good_comp    << '\n';
+        out << "@ arma_config::extra_code   = " << arma_config::extra_code   << '\n';
+        out << "@ arma_config::mat_prealloc = " << arma_config::mat_prealloc << '\n';
         out << "@ sizeof(void*)    = " << sizeof(void*)    << '\n';
         out << "@ sizeof(uword)    = " << sizeof(uword)    << '\n';
         out << "@ sizeof(int)      = " << sizeof(int)      << '\n';
