@@ -1,18 +1,10 @@
-// Copyright (C) 2010 NICTA and the authors listed below
-// http://nicta.com.au
+// Copyright (C) 2009-2010 Conrad Sanderson
+// Copyright (C) 2009-2010 NICTA (www.nicta.com.au)
+// Copyright (C) 2009-2010 Dimitrios Bouzas
 // 
-// Authors:
-// - Conrad Sanderson (conradsand at ieee dot org)
-// - Dimitrios Bouzas (dimitris dot mpouzas at gmail dot com)
-// 
-// This file is part of the Armadillo C++ library.
-// It is provided without any warranty of fitness
-// for any purpose. You can redistribute this file
-// and/or modify it under the terms of the GNU
-// Lesser General Public License (LGPL) as published
-// by the Free Software Foundation, either version 3
-// of the License or (at your option) any later version.
-// (see http://www.opensource.org/licenses for more info)
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 
 
@@ -26,11 +18,10 @@
 template<typename T1>
 inline
 const eOp<T1, eop_eps>
-eps(const Base<typename T1::elem_type, T1>& X)
+eps(const Base<typename T1::elem_type, T1>& X, const typename arma_not_cx<typename T1::elem_type>::result* junk = 0)
   {
   arma_extra_debug_sigprint();
-  
-  typedef typename T1::elem_type eT;
+  arma_ignore(junk);
   
   return eOp<T1, eop_eps>(X.get_ref());
   }
@@ -42,10 +33,12 @@ eps(const Base<typename T1::elem_type, T1>& X)
 template<typename T1>
 inline
 Mat< typename T1::pod_type >
-eps(const Base< std::complex<typename T1::pod_type>, T1>& X)
+eps(const Base< std::complex<typename T1::pod_type>, T1>& X, const typename arma_cx_only<typename T1::elem_type>::result* junk = 0)
   {
   arma_extra_debug_sigprint();
   
+  arma_ignore(junk);
+  
   typedef typename T1::pod_type   T;
   typedef typename T1::elem_type eT;
   
@@ -56,9 +49,9 @@ eps(const Base< std::complex<typename T1::pod_type>, T1>& X)
   
          T* out_mem = out.memptr();
   const eT* A_mem   = A.memptr();
-  const u32 n_elem  = A.n_elem;
+  const uword n_elem  = A.n_elem;
   
-  for(u32 i=0; i<n_elem; ++i)
+  for(uword i=0; i<n_elem; ++i)
     {
     out_mem[i] = eop_aux::direct_eps( A_mem[i] );
     }
@@ -75,6 +68,8 @@ arma_warn_unused
 typename arma_integral_only<eT>::result
 eps(const eT& x)
   {
+  arma_ignore(x);
+  
   return eT(0);
   }
 
@@ -83,7 +78,7 @@ eps(const eT& x)
 template<typename eT>
 arma_inline
 arma_warn_unused
-typename arma_float_only<eT>::result
+typename arma_real_only<eT>::result
 eps(const eT& x)
   {
   return eop_aux::direct_eps(x);
@@ -94,7 +89,7 @@ eps(const eT& x)
 template<typename T>
 arma_inline
 arma_warn_unused
-typename arma_float_only<T>::result
+typename arma_real_only<T>::result
 eps(const std::complex<T>& x)
   {
   return eop_aux::direct_eps(x);
