@@ -1,17 +1,9 @@
-// Copyright (C) 2010 NICTA and the authors listed below
-// http://nicta.com.au
+// Copyright (C) 2010-2013 Conrad Sanderson
+// Copyright (C) 2010-2013 NICTA (www.nicta.com.au)
 // 
-// Authors:
-// - Conrad Sanderson (conradsand at ieee dot org)
-// 
-// This file is part of the Armadillo C++ library.
-// It is provided without any warranty of fitness
-// for any purpose. You can redistribute this file
-// and/or modify it under the terms of the GNU
-// Lesser General Public License (LGPL) as published
-// by the Free Software Foundation, either version 3
-// of the License or (at your option) any later version.
-// (see http://www.opensource.org/licenses for more info)
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 
 //! \addtogroup fn_join
@@ -31,6 +23,18 @@ join_cols(const Base<typename T1::elem_type,T1>& A, const Base<typename T1::elem
 
 
 
+template<typename T1, typename T2>
+inline
+const Glue<T1, T2, glue_join>
+join_vert(const Base<typename T1::elem_type,T1>& A, const Base<typename T1::elem_type,T2>& B)
+  {
+  arma_extra_debug_sigprint();
+  
+  return Glue<T1, T2, glue_join>(A.get_ref(), B.get_ref(), 0);
+  }
+
+
+
 template<typename T1, typename T2>
 inline
 const Glue<T1, T2, glue_join>
@@ -43,4 +47,28 @@ join_rows(const Base<typename T1::elem_type,T1>& A, const Base<typename T1::elem
 
 
 
+template<typename T1, typename T2>
+inline
+const Glue<T1, T2, glue_join>
+join_horiz(const Base<typename T1::elem_type,T1>& A, const Base<typename T1::elem_type,T2>& B)
+  {
+  arma_extra_debug_sigprint();
+  
+  return Glue<T1, T2, glue_join>(A.get_ref(), B.get_ref(), 1);
+  }
+
+
+
+template<typename T1, typename T2>
+inline
+const GlueCube<T1, T2, glue_join>
+join_slices(const BaseCube<typename T1::elem_type,T1>& A, const BaseCube<typename T1::elem_type,T2>& B)
+  {
+  arma_extra_debug_sigprint();
+  
+  return GlueCube<T1, T2, glue_join>(A.get_ref(), B.get_ref());
+  }
+
+
+
 //! @}
