@@ -1,14 +1,9 @@
-// Copyright (C) 2008-2012 NICTA (www.nicta.com.au)
 // Copyright (C) 2008-2012 Conrad Sanderson
+// Copyright (C) 2008-2012 NICTA (www.nicta.com.au)
 // 
-// This file is part of the Armadillo C++ library.
-// It is provided without any warranty of fitness
-// for any purpose. You can redistribute this file
-// and/or modify it under the terms of the GNU
-// Lesser General Public License (LGPL) as published
-// by the Free Software Foundation, either version 3
-// of the License or (at your option) any later version.
-// (see http://www.opensource.org/licenses for more info)
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 
 //! \addtogroup podarray
@@ -209,6 +204,21 @@ podarray<eT>::operator() (const uword i)
 
 
 
+template<typename eT>
+inline
+void
+podarray<eT>::set_min_size(const uword min_n_elem)
+  {
+  arma_extra_debug_sigprint();
+  
+  if(min_n_elem > n_elem)
+    {  
+    init_warm(min_n_elem);
+    }
+  }
+
+
+
 template<typename eT>
 inline
 void
@@ -252,7 +262,7 @@ podarray<eT>::zeros()
   {
   arma_extra_debug_sigprint();
   
-  fill(eT(0));
+  arrayops::fill_zeros(memptr(), n_elem);
   }
 
 
@@ -266,7 +276,7 @@ podarray<eT>::zeros(const uword new_n_elem)
   
   init_warm(new_n_elem);
   
-  fill(eT(0));
+  arrayops::fill_zeros(memptr(), n_elem);
   }
 
 
@@ -323,34 +333,19 @@ podarray<eT>::copy_row(const Mat<eT>& A, const uword row)
       }
       break;
     
-    case 8:
-      out[7] = A.at(row, 7);
-
-    case 7:
-      out[6] = A.at(row, 6);
-
-    case 6:
-      out[5] = A.at(row, 5);
-
-    case 5:
-      out[4] = A.at(row, 4);
-
-    case 4:
-      out[3] = A.at(row, 3);
-
-    case 3:
-      out[2] = A.at(row, 2);
-
-    case 2:
-      out[1] = A.at(row, 1);
-
-    case 1:
-      out[0] = A.at(row, 0);
+    case 8:  out[7] = A.at(row, 7);
+    case 7:  out[6] = A.at(row, 6);
+    case 6:  out[5] = A.at(row, 5);
+    case 5:  out[4] = A.at(row, 4);
+    case 4:  out[3] = A.at(row, 3);
+    case 3:  out[2] = A.at(row, 2);
+    case 2:  out[1] = A.at(row, 1);
+    case 1:  out[0] = A.at(row, 0);
+    case 0:  ;
     }
   }
 
 
-
 template<typename eT>
 arma_hot
 inline
@@ -366,8 +361,6 @@ podarray<eT>::init_cold(const uword new_n_elem)
   else
     {
     mem = memory::acquire<eT>(new_n_elem);
-    
-    arma_check_bad_alloc( (mem == 0), "arma::podarray: out of memory" );
     }
   }
 
@@ -397,8 +390,6 @@ podarray<eT>::init_warm(const uword new_n_elem)
   else
     {
     mem = memory::acquire<eT>(new_n_elem);
-    
-    arma_check_bad_alloc( (mem == 0), "arma::podarray: out of memory" );
     }
   
   access::rw(n_elem) = new_n_elem;
