@@ -1,16 +1,9 @@
-// Copyright (C) 2009 NICTA
+// Copyright (C) 2009-2011 Conrad Sanderson
+// Copyright (C) 2009-2011 NICTA (www.nicta.com.au)
 // 
-// Authors:
-// - Conrad Sanderson (conradsand at ieee dot org)
-// 
-// This file is part of the Armadillo C++ library.
-// It is provided without any warranty of fitness
-// for any purpose. You can redistribute this file
-// and/or modify it under the terms of the GNU
-// Lesser General Public License (LGPL) as published
-// by the Free Software Foundation, either version 3
-// of the License or (at your option) any later version.
-// (see http://www.opensource.org/licenses for more info)
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 
 //! \addtogroup running_stat
@@ -30,8 +23,8 @@ arma_counter<eT>::~arma_counter()
 template<typename eT>
 inline
 arma_counter<eT>::arma_counter()
-  : d_count( eT(0))
-  , i_count(u32(0))
+  : d_count(   eT(0))
+  , i_count(uword(0))
   {
   arma_extra_debug_sigprint_this(this);
   }
@@ -43,16 +36,14 @@ inline
 const arma_counter<eT>& 
 arma_counter<eT>::operator++()
   {
-  const u32 max_val = 0xffffffff;
-  
-  if(i_count < max_val)
+  if(i_count < ARMA_MAX_UWORD)
     {
     i_count++;
     }
   else
     {
-    d_count += eT(max_val);
-    i_count =  1;
+    d_count += eT(ARMA_MAX_UWORD);
+    i_count  = 1;
     }
   
   return *this;
@@ -75,8 +66,8 @@ inline
 void
 arma_counter<eT>::reset()
   {
-  d_count =  eT(0);
-  i_count = u32(0);
+  d_count =    eT(0);
+  i_count = uword(0);
   }
 
 
@@ -84,8 +75,7 @@ arma_counter<eT>::reset()
 template<typename eT>
 inline
 eT
-arma_counter<eT>::value()
-const
+arma_counter<eT>::value() const
   {
   return d_count + eT(i_count);
   }
@@ -95,18 +85,15 @@ const
 template<typename eT>
 inline
 eT
-arma_counter<eT>::value_plus_1()
-const
+arma_counter<eT>::value_plus_1() const
   {
-  const u32 max_val = 0xffffffff;
-  
-  if(i_count < max_val)
+  if(i_count < ARMA_MAX_UWORD)
     {
     return d_count + eT(i_count + 1);
     }
   else
     {
-    return d_count + eT(max_val) + eT(1);
+    return d_count + eT(ARMA_MAX_UWORD) + eT(1);
     }
   }
 
@@ -115,8 +102,7 @@ const
 template<typename eT>
 inline
 eT
-arma_counter<eT>::value_minus_1()
-const
+arma_counter<eT>::value_minus_1() const
   {
   if(i_count > 0)
     {
@@ -164,8 +150,12 @@ running_stat<eT>::operator() (const typename running_stat<eT>::T sample)
   {
   arma_extra_debug_sigprint();
   
-  arma_check( (arma_isfinite(sample) == false), "running_stat: non-finite sample given" );
-
+  if( arma_isfinite(sample) == false )
+    {
+    arma_warn(true, "running_stat: sample ignored as it is non-finite" );
+    return;
+    }
+  
   running_stat_aux::update_stats(*this, sample);
   }
 
@@ -179,9 +169,11 @@ running_stat<eT>::operator() (const std::complex< typename running_stat<eT>::T >
   {
   arma_extra_debug_sigprint();
   
-  isnt_same_type<eT, std::complex< typename running_stat<eT>::T > >::check();
-  
-  arma_check( (arma_isfinite(sample) == false), "running_stat: non-finite sample given" );
+  if( arma_isfinite(sample) == false )
+    {
+    arma_warn(true, "running_stat: sample ignored as it is non-finite" );
+    return;
+    }
   
   running_stat_aux::update_stats(*this, sample);
   }
@@ -196,7 +188,7 @@ running_stat<eT>::reset()
   {
   arma_extra_debug_sigprint();
   
-  typedef typename running_stat<eT>::T T;
+  // typedef typename running_stat<eT>::T T;
   
   counter.reset();
   
@@ -216,8 +208,7 @@ running_stat<eT>::reset()
 template<typename eT>
 inline
 eT
-running_stat<eT>::mean()
-const
+running_stat<eT>::mean() const
   {
   arma_extra_debug_sigprint();
   
@@ -230,8 +221,7 @@ const
 template<typename eT>
 inline
 typename running_stat<eT>::T
-running_stat<eT>::var(const u32 norm_type)
-const
+running_stat<eT>::var(const uword norm_type) const
   {
   arma_extra_debug_sigprint();
   
@@ -261,11 +251,10 @@ const
 template<typename eT>
 inline
 typename running_stat<eT>::T
-running_stat<eT>::stddev(const u32 norm_type)
-const
+running_stat<eT>::stddev(const uword norm_type) const
   {
   arma_extra_debug_sigprint();
-
+  
   return std::sqrt( (*this).var(norm_type) );
   }
 
@@ -275,8 +264,7 @@ const
 template<typename eT>
 inline
 eT
-running_stat<eT>::min()
-const
+running_stat<eT>::min() const
   {
   arma_extra_debug_sigprint();
   
@@ -289,23 +277,36 @@ const
 template<typename eT>
 inline
 eT
-running_stat<eT>::max()
-const
+running_stat<eT>::max() const
   {
   arma_extra_debug_sigprint();
-
+  
   return max_val;
   }
 
 
 
-//! update statistics to reflect new sample
+//! number of samples so far
+template<typename eT>
+inline
+typename get_pod_type<eT>::result
+running_stat<eT>::count() const
+  {
+  arma_extra_debug_sigprint();
+  
+  return counter.value();
+  }
+
+
+
+//! update statistics to reflect new sample (version for non-complex numbers, non-complex sample)
 template<typename eT>
 inline
 void
-running_stat_aux::update_stats(running_stat<eT>& x, const eT sample)
+running_stat_aux::update_stats(running_stat<eT>& x, const eT sample, const typename arma_not_cx<eT>::result* junk)
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk);
   
   typedef typename running_stat<eT>::T T;
   
@@ -351,28 +352,46 @@ running_stat_aux::update_stats(running_stat<eT>& x, const eT sample)
 
 
 
-//! update statistics to reflect new sample (version for complex numbers)
-template<typename T>
+//! update statistics to reflect new sample (version for non-complex numbers, complex sample)
+template<typename eT>
 inline
 void
-running_stat_aux::update_stats(running_stat< std::complex<T> >& x, const T sample)
+running_stat_aux::update_stats(running_stat<eT>& x, const std::complex<eT>& sample, const typename arma_not_cx<eT>::result* junk)
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  running_stat_aux::update_stats(x, std::real(sample));
+  }
 
+
+
+//! update statistics to reflect new sample (version for complex numbers, non-complex sample)
+template<typename eT>
+inline
+void
+running_stat_aux::update_stats(running_stat<eT>& x, const typename eT::value_type sample, const typename arma_cx_only<eT>::result* junk)
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  typedef typename eT::value_type T;
+  
   running_stat_aux::update_stats(x, std::complex<T>(sample));
   }
 
 
 
-//! alter statistics to reflect new sample (version for complex numbers)
-template<typename T>
+//! alter statistics to reflect new sample (version for complex numbers, complex sample)
+template<typename eT>
 inline
 void
-running_stat_aux::update_stats(running_stat< std::complex<T> >& x, const std::complex<T>& sample)
+running_stat_aux::update_stats(running_stat<eT>& x, const eT& sample, const typename arma_cx_only<eT>::result* junk)
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk);
   
-  typedef typename std::complex<T> eT;
+  typedef typename eT::value_type T;
   
   const T sample_norm = std::norm(sample);
   const T N           = x.counter.value();
